#include <linuxmt/types.h>
#include <linuxmt/config.h>
#include <linuxmt/sched.h>
#include <linuxmt/errno.h>
#include <arch/system.h>
#include <linuxmt/fs.h>
#include <linuxmt/genhd.h>

#define MAJOR_NR	BIOSHD_MAJOR
#include  "blk.h"

#ifdef CONFIG_BIOS_HD

struct hd_info
{
	unsigned short cyl;
	unsigned short wpcom;
	unsigned short lzone;
	unsigned char head;
	unsigned char sect;
	unsigned char ctl;
};

static struct hd_info disks[2];

/*
 *	Just don't re-enter 8). Easy ways to get the parameters to
 *	the #asm bits.. should use 2[bp] etc - FIXME
 */
 
static unsigned int hd_unit;  /* BIOS xt drive (drive + 0x80) */
static unsigned int hd_stat; /* XT status */
static unsigned int hd_cylh; /* BIOS cylender */
static unsigned int hd_cyll; /* BIOS cylender */
static unsigned int hd_head; /* BIOS head */
static unsigned int hd_sector; /* BIOS sector */

static int hd_seek(hd_sec, hd_hed, hd_cyl, hd_dsk)
unsigned int hd_sec;
unsigned int hd_hed;
unsigned int hd_cyl;
unsigned int hd_dsk;
{
	hd_unit=hd_dsk + 0x80;
	hd_head = hd_hed & 0x0f; /* head */
	hd_cylh=((hd_cyl / 256) << 6) | hd_sec;    /* high 2 bits + sector */
	hd_cyll=hd_cyl & 0x00ff; /* low 8 bits */
/* printk("%d %d %d %d\r",hd_hed,hd_cyl,hd_cylh, hd_cyll); */

	/*
	 *	perform seek operation, return status 
	 */
	 
#asm
	mov	ah,#0x0c
	mov	cl,_hd_cylh
	mov     ch,_hd_cyll
	mov	dh,_hd_head
	mov     dl,_hd_unit
	int	0x13
	mov     _hd_stat,ah
#endasm

	/* printk("%d ",hd_stat); */
	return(hd_stat);
}

/*
 *	Recalibrate and recover the drive.
 */
 
static int hd_recal(hd_dsk)
unsigned int hd_dsk;
{
	hd_unit = hd_dsk + 0x80;
	/*
	 *	Re-calibrate the drive (home heads) 
	 */
#asm
	mov	ah,#0x11
	mov	dl,_hd_unit
	int	0x13		
	mov	_hd_stat,ah
#endasm
	return(hd_stat);
}

static int hd_read(hd_dsk, head, cyl, sec)
unsigned int hd_dsk;
unsigned int head;
unsigned int cyl;
unsigned int sec;
{
}

static int hd_write(hd_dsk, head, cyl, sec)
unsigned int hd_dsk;
unsigned int head;
unsigned int cyl;
unsigned int sec;
{
}



/*
 *	Control structures
 */

static struct file_operations bioshd_fops = {
	NULL,			/* lseek - default */
	block_read,		/* read - general block-dev read */
	block_write,		/* write - general block-dev write */
	NULL,			/* readdir - bad */
	NULL,			/* select */
	bioshd_ioctl,		/* ioctl */
	NULL,			/* mmap */
	bioshd_open,		/* open */
	bioshd_release,		/* release */
	block_fsync		/* fsync */
};

static struct gendisk bioshd_gendisk = 
{
	MAJOR_NR,	/* Major number */
	"bioshd",	/* Major name */
	6,		/* Bits to shift to get real from partition */
	1 << 6,		/* Number of partitions per real */
	2,		/* maximum number of real */
	bioshd_geninit,	/* init function */
	bioshd,		/* hd struct */
	bioshd_sizes,	/* block sizes */
	0,		/* number */
	NULL,		/* internal */
	NULL		/* next */
};

/*
 *	Initialise the hard disk controller.
 */
 
void hd_init()
{
	
	int d=0;
	int pos=0x80;	/* Disk base */
	unsigned long size;

	if(register_blkdev(MAJOR_NR,"bioshd", &bhd_fops))
	{
		printk("biosh_init: unable to register device %d.\n", MAJOR_NR);
		return;
	}
	
	blk_dev[MAJOR_NR].request_fn = DEVICE_REQUEST;
	read_ahead[MAJOR_NR] = 2;
	bioshd_gendisk.next = gendisk_head;
	gendisk_head = bioshd_gendisk;
	for(d=0;d<2;d++)
	{
		/*
		 *	Pull the AT settings
		 */
		 
		disks[d].cyl	=	peekw(0x9000,pos);
		disks[d].head	=	peekb(0x9000,pos+2);
		disks[d].wpcom	=	peekw(0x9000,pos+5);
		disks[d].ctl	=	peekb(0x9000,pos+8);
		disks[d].lzone	=	peekw(0x9000,pos+12);
		disks[d].sect	=	peekb(0x9000,pos+14);
		
		pos+=16;
		if(disks[d].cyl && disks[d].sect)
		{
			/* Size in 1K blocks */
			size=disks[d].head*disks[d].sect*(disks[d].cyl/2);
			/* Convert to Mb */
			size=size / 1024;
			printk("hd%c: %d heads %d cylinders, %d sectors (%ldMb).\n",
				d+'a',disks[d].head,disks[d].cyl,disks[d].sect, size);
		}
		
	}
}	
#endif
